"""
Copyright 2012 NetApp, Inc. All Rights Reserved,
contribution by Weston Andros Adamson <dros@netapp.com>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
"""

import os, posix, stat, sys

from tags import *

CONFIG_MTIME=os.stat(__file__)[stat.ST_MTIME]

NFSOMETER_VERSION='1.1'

RUNNING_TRACE_DIR='/tmp/nfsometer_trace'
TRACE_ATTRFILE='arguments'
TRACE_DIR_PREFIX='nfsometer_trace'
TRACE_DIR_VERSION=3

TRACE_MOUNT_TRIES = 3
TRACE_MOUNT_TRY_DELAY = 0.5
TRACE_UMOUNT_TRIES = 3
TRACE_CLEANUP_UMOUNT_TRIES = 15
TRACE_UMOUNT_TRY_DELAY = 0.5

MOUNTOPTS_V2   = 'vers=2'
MOUNTOPTS_V3   = 'vers=3'
MOUNTOPTS_V4   = 'vers=4'
MOUNTOPTS_V4_1 = 'vers=4,minorversion=1'

MOUNTOPTS_ALIASES = {
    'v2':   MOUNTOPTS_V2,
    'v3':   MOUNTOPTS_V3,
    'v4.0': MOUNTOPTS_V4,
    'v4.1': MOUNTOPTS_V4_1,
}

def normalize_mountopts(mountopts):
    new = []
    for mo in mountopts:
        entry = []
        for part in mo.split(','):
            if MOUNTOPTS_ALIASES.has_key(part):
                entry.extend(MOUNTOPTS_ALIASES[part].split(','))
            else:
                entry.append(part)

        new.append(','.join(entry))

    return new

NFSOMETERLIB_DIR=os.path.split(__file__)[0]
NFSOMETER_DIR=os.path.join(posix.environ['HOME'], '.nfsometer')
HTML_DIR="%s/html" % NFSOMETERLIB_DIR

# locations
MOUNTDIR='/mnt'
WORKLOADFILES_ROOT=os.path.join(NFSOMETER_DIR, 'workload_files')
RESULTS_DIR=os.path.join(posix.environ['HOME'], 'nfsometer_results')
RUNROOT='%s/nfsometer_runroot' % MOUNTDIR

TEMPLATE_TOC='%s/toc.html' % HTML_DIR
TEMPLATE_TOCNODE='%s/tocnode.html' % HTML_DIR
TEMPLATE_TABLE='%s/table.html' % HTML_DIR
TEMPLATE_DATASET='%s/dataset.html' % HTML_DIR
TEMPLATE_DATASETKEYINFO='%s/dataset_key_info.html' % HTML_DIR
TEMPLATE_WIDGET='%s/widget.html' % HTML_DIR
TEMPLATE_SECTION='%s/section.html' % HTML_DIR
TEMPLATE_REPORT='%s/report.html' % HTML_DIR
TEMPLATE_INDEX='%s/index.html' % HTML_DIR
TEMPLATE_REPORTLIST='%s/reportlist.html' % HTML_DIR
TEMPLATE_CELLDATAINFO='%s/cell_data_info.html' % HTML_DIR
TEMPLATE_LOADINFO='%s/load_info.html' % HTML_DIR

CSSFILEPATH='%s/style.css' % HTML_DIR
JSFILEPATH='%s/script.js' % HTML_DIR
JQUERY_URL='http://code.jquery.com/jquery-1.7.2.min.js'

GRAPH_ERRORBAR_COLOR='#CF2727'
GRAPH_ERRORBAR_WIDTH=1
GRAPH_EDGE_COLOR='#000000'

FILE_COMMAND='command.sh'

_NFSOPTS_DISPLAY_MAP = {
    make_nfsopt(MOUNTOPTS_V2,   None):
        'v2',
    make_nfsopt(MOUNTOPTS_V2,   nfstags(TAG_REMOTE)):
        'v2 remote',
    make_nfsopt(MOUNTOPTS_V3,   None):
        'v3',
    make_nfsopt(MOUNTOPTS_V3,   nfstags(TAG_REMOTE)):
        'v3 remote',
    make_nfsopt(MOUNTOPTS_V4,   None):
        'v4',
    make_nfsopt(MOUNTOPTS_V4,   nfstags(TAG_REMOTE)):
        'v4 remote',
    make_nfsopt(MOUNTOPTS_V4,   nfstags(TAG_DELEG_ENABLED)):
        'v4 deleg',
    make_nfsopt(MOUNTOPTS_V4,   nfstags(TAG_DELEG_ENABLED, TAG_REMOTE)):
        'v4 deleg remote',
    make_nfsopt(MOUNTOPTS_V4_1, None):
        'v4.1',
    make_nfsopt(MOUNTOPTS_V4_1, nfstags(TAG_REMOTE)):
        'v4.1 remote',
    make_nfsopt(MOUNTOPTS_V4_1, nfstags(TAG_PNFS_ENABLED)):
        'v4.1 pnfs',
    make_nfsopt(MOUNTOPTS_V4_1, nfstags(TAG_PNFS_ENABLED, TAG_REMOTE)):
        'v4.1 pnfs remote',
}

COLOR_TERMS = ('xterm-color', 'screen')

def highlight(msg):
    if posix.environ['TERM'] in COLOR_TERMS:
        pre = '\033[33m'
        post = '\033[0m'
    else:
        pre = '>> '
        post = ''

    print "%s%s%s" % (pre, msg, post)

def inform(msg):
    if posix.environ['TERM'] in COLOR_TERMS:
        pre = '\033[32m'
        post = '\033[0m'
    else:
        pre = '>> '
        post = ''

    print "%s%s%s" % (pre, msg, post)

def warn(msg):
    if posix.environ['TERM'] in COLOR_TERMS:
        pre = '\033[41m'
        post = '\033[0m'
    else:
        pre = 'WARNING> '
        post = ''

    print "%s%s%s" % (pre, msg, post)

def import_error(m):
     print >>sys.stderr, m
     sys.exit(1)


WORKLOADS_DIR=os.path.join(NFSOMETERLIB_DIR, 'workloads')
WORKLOADS_SCRIPT=os.path.join(WORKLOADS_DIR, 'workload.sh')

def get_workload_display(w):
    if isinstance(w, (list, tuple)):
        return ' vs. '.join([ get_workload_display(x) for x in w ])
    return w

def get_nfsopt_display(nfsopt):
    if isinstance(nfsopt, (list, tuple)):
        return ' vs. '.join([ get_nfsopt_display(x) for x in nfsopt ])

    if _NFSOPTS_DISPLAY_MAP.has_key(nfsopt):
        return _NFSOPTS_DISPLAY_MAP[nfsopt]

    # try to split off client args
    mt, st, ct = nfsopt_split(nfsopt)
    lookup = ','.join(mt)
    if st:
        lookup += ',' + ','.join(st)

    if _NFSOPTS_DISPLAY_MAP.has_key(lookup):
        display = _NFSOPTS_DISPLAY_MAP[lookup]

        if ct:
            display += ' ' + ', '.join(ct)

        return display

    return nfsopt

def get_kernel_display(kernel):
    if isinstance(kernel, (list, tuple)):
        return ' vs. '.join([ get_kernel_display(x) for x in kernel ])
    return kernel

#
# NOTE_* - disclaimers and such
#

# TODO this should be based off of some arg in workload def
#      for fixed-time tests
def note_filebench_times(sel):
    has_fb = False

    for w in sel.workloads:
        if w.startswith('filebench_'):
            has_fb = True
            break

    if has_fb:
        return """Filebench tests are run for a set amount
                  of time the <i>time_real</i> value is somewhat
                  useless.
               """
    return ''

def note_v3_no_lock(sel):
    if sel.has_mountopt(MOUNTOPTS_V3):
        return """NFSv3's locking protocol runs on a different service
                  and is not counted
               """
    return ''

def note_v41_pnfs_no_ds(sel):
    old_kernel = False
    for kernel in sel.kernels:
        # XXX a hack, and not really true since some versions < 3 do have these
        #     stats
        if kernel.startswith('2.'):
            old_kernel = True
            break

    if old_kernel:
        return """ Older linux kernels do not count READ, WRITE and
                   COMMIT operations to pNFS dataservers (unless the
                   DS is also the MDS)."""
    return ''
 
# keyed by [test_name][stat_name]
TEST_BOUND_UNKNOWN = 0
TEST_BOUND_IO = 1
TEST_BOUND_TIME = 2

BETTER_UNKNOWN = 0
BETTER_ALWAYS_LESS = 1
BETTER_ALWAYS_MORE = 2
BETTER_LESS_IF_IO_BOUND = 3 # but more if time bound
BETTER_MORE_IF_IO_BOUND = 4 # but less if time bound

#WIDE_DATA_THRESHOLD = 20
WIDE_DATA_THRESHOLD = 6

def better_info(bounds, better):
    if better == BETTER_ALWAYS_LESS:
        less_is_better = True

    elif better == BETTER_ALWAYS_MORE:
        less_is_better = False

    elif better == BETTER_LESS_IF_IO_BOUND:
        if bounds == TEST_BOUND_IO:
            less_is_better = True
        else:
            less_is_better = False

    elif better == BETTER_MORE_IF_IO_BOUND:
        if bounds == TEST_BOUND_IO:
            less_is_better = False
        else:
            less_is_better = True

    else:
        return ('', '')

    if less_is_better:
        return ('&darr;', 'less is better')
    else:
        return ('&uarr;', 'more is better')

CONST_TIME_EXCUSE = " as this test is time constrained"

def get_tracedir(resdir, test, nfsopt):
    tracedir_root = '%s-%s-%s' % (TRACE_DIR_PREFIX, test, nfsopt)

    i = 0
    while True:
        tracedir = '%s-%s' % (tracedir_root, i)

        if not os.path.exists(os.path.join(resdir, tracedir)):
            return tracedir

        i += 1

_NEW_COLORS=[
  '#FEC44F', # Yellow
  '#D95F0E', # Orange
  '#476CDA', # Blue
  '#336600', # Green
  '#008B8B', # Turquoise
  '#000000', # Black
  '#6B24B2', # Dark Purple
  '#1E90FF', # Light Blue
  '#94704D', # Brown
  '#FF4D94', # Pink
  '#666600', # Olive
  '#B82E8A', # Purple
  '#85A366', # Light Green
  '#BC8F8F', # Light Purple
  '#FF0066', # Dark Pink
  '#D63030', # Red
]


COLORS = [
  '#FEC44F', # Yellow
  '#D95F0E', # Orange
  '#476CDA', # Blue
  '#336600', # Green
  '#008B8B', # Turquoise
  '#303030', # Blackish

# --
#  '#7F3B08',
#  '#B35806',
#  '#E08214',
#  '#FDB863',
  '#FEE0B6',
#  '#D8DAEB',
  '#B2ABD2',
  '#8073AC',
  '#542788',
  '#2D004B',

# --
  '#67001F',
  '#B2182B',
  '#D6604D',
  '#F4A582',
  '#FDDBC7',
  '#E0E0E0',
  '#BABABA',
  '#878787',
  '#4D4D4D',
  '#1A1A1A',
]

_OTHER_COLORS = [
# colorset 1
  '#A6CEE3',
  '#1F78B4',
  '#B2DF8A',
  '#33A02C',
  '#FB9A99',
  '#E31A1C',
  '#FDBF6F',
  '#FF7F00',
  '#CAB2D6',
  '#6A3D9A',

# colorset 2
  '#8DD3C7',
  '#FFFFB3',
  '#BEBADA',
  '#FB8072',
  '#80B1D3',
  '#FDB462',
  '#B3DE69',
  '#FCCDE5',
  '#D9D9D9',
  '#BC80BD',

]

HATCHES = ['/', '.', 'x', '|', 'o', '-', '+', '\\', 'O', '*', ]

def hatch_idx(i):
    assert i > 0
    return (i - 1) % len(HATCHES)

def get_hatch(i):
    if i > 0:
        return HATCHES[hatch_idx(i)]
    return ''

def get_hatch_html(i, title=''):
    hatch = get_hatch(i)
    if not hatch:
        hatch = '&nbsp;'
    return '<span class="hatch" title="%s">%s</span>' % (title, hatch,)

def color_idx(i):
    return i % len(COLORS)

try:
    import numpy as np
except:
    import_error("Error importing numpy - Make sure numpy is installed")

def _check_mpl_version(vers):
    warning = False
    sv = vers.split('.')

    if int(sv[0]) < 1:
        warning = True
    elif int(sv[0]) == 1 and int(sv[1]) < 1:
        warning = True

    if warning:
        warn("matplotlib version %s < 1.1 - some graph features might not work!!" % vers)


try:
    import matplotlib
except:
    import_error("Error importing matplotlib - Make sure matplotlib is installed")

_check_mpl_version(matplotlib.__version__)

try:
    # Don't require $DISPLAY to be set!
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import matplotlib.font_manager as fm
except:
    import_error("Error importing matplotlib submodules - this is probably an incompatible version of matplotlib")

try:
    from mako.template import Template
except:
    import_error("Error importing mako - Make sure mako is installed")

PLUSMINUS = '&#177;'

NO_DATA_HTML='<span class="no_data">no data</span>'
COMPARISON_ZERO_HTML='<span class="zero_data">none</span>'

def pluralize(x, pluralstr='s'):
    if x != 1:
        return pluralstr
    return ''


