#!/bin/bash
#
# Copyright 2012 NetApp, Inc. All Rights Reserved,
# contribution by Weston Andros Adamson <dros@netapp.com>
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#

if [ $# -ne 3 ] ; then
    echo "usage: $0 <adminserver> <setting> <check|enabled|disabled>"
    echo "  where setting is:"
    echo "   deleg_v4.0"
    echo "   pnfs_v4.1"
    exit 1
fi

# XXX move to argument 
ADMINSERVER="$1"
SETTING="$2"
CMD="$3"

_rcmd() {
    ssh -t -t -q "$ADMINSERVER" $*
}

_health_check() {
    # check for network interfaces that are not home
    if [ -n "$(_rcmd network int show | grep up | grep false)" ] ; then
        echo "migrated lif"
        return
    fi
}

_deleg_check() {
    _read=$(_rcmd nfs show -instance | grep "NFSv4.0 Read Delegation" | cut -f2 -d: | sed 's/ //g')
    _write=$(_rcmd nfs show -instance | grep "NFSv4.0 Write Delegation" | cut -f2 -d: | sed 's/ //g')

    if [ "$_read" = "$_write" ]; then
        echo "$_read"
    else
        echo "ERROR!"
    fi
}

_deleg_mod() {
    _rcmd "nfs modify -v4.0-read-delegation $1"
    _rcmd "nfs modify -v4.0-write-delegation $1"
}

_pnfs_check() {
        _rcmd "nfs show -instance" | grep "NFSv4.1 Parallel NFS" | cut -f2 -d: | sed 's/ //g'
}

_pnfs_mod() {
        _rcmd "nfs modify -v4.1-pnfs $1" > /dev/null
}

if [ "$SETTING" = "deleg_v4.0" ]; then
    if [ "$CMD" = "check" ] ; then
        _deleg_check
    elif [ "$CMD" = "enabled" ] ; then
        _deleg_mod "$CMD"
    elif [ "$CMD" = "disabled" ] ; then
        _deleg_mod "$CMD"
    fi

elif [ "$SETTING" = "pnfs_v4.1" ]; then
    if [ "$CMD" = "check" ] ; then
        _pnfs_check
    elif [ "$CMD" = "enabled" ] ; then
        _pnfs_mod $CMD
    elif [ "$CMD" = "disabled" ] ; then
        _pnfs_mod $CMD
    fi

elif [ "$SETTING" = "health" ] ; then
    if [ "$CMD" = "check" ] ; then
        _health_check
    fi
fi
