"""
Copyright 2012 NetApp, Inc. All Rights Reserved,
contribution by Weston Andros Adamson <dros@netapp.com>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
"""

from config import *
from cmd import *

class ClientAdmin:
    def __init__(self, script, args):
        """ args - list of "name=val1,val2,val3" strings """
        self.script = script
        self.args = args

    def all_tag_combinations(self):
        all_tags = []

        for arg in self.args:
            name, values = arg.split('=', 1)
            values = values.split(',')

            new = []

            for v in values:
                new.append('%s=%s' % (name, v))

            if not all_tags:
                all_tags = new
            else:
                # combine all_tags and new
                comb = []
                for a in all_tags:
                    for n in new:
                        comb.append(','.join((a,n)))
                all_tags = comb

        return all_tags

    def handle_settings(self, mountopt, tags):
        if not tags:
            tags = []

        for tag in tags:
            if tag in TAG_DESC:
                continue

            name, value = tag.split('=', 1)

            print "ClientAdmin: Setting %s -> %s" % (name, value)

            cmd('%s set %s %s' % (self.script, name, value),
                raiseerrorout=False)
            res = cmd('%s get %s' % (self.script, name),
                raiseerrorout=False)
            res = '\n'.join(res[0])

            if res.strip() != value:
                raise Exception("ClientAdmin %s: failed to set %s" %
                        (self.script, need))


class ServerAdmin:
    def __init__(self, script, server):
        self.script = script
        self.server = server

        if self.script:
            assert self.server, "script specified, but no server"

        self.settings = {
            (MOUNTOPTS_V4, TAG_DELEG_ENABLED):  'deleg_v4.0',
            (MOUNTOPTS_V4_1, TAG_PNFS_ENABLED): 'pnfs_v4.1',
        }

        self._check_support()

    def _check_support(self):
        keys = self.settings.keys()
        for mountopt, tag in keys:
            setting = self.settings[(mountopt, tag)]
            supported = False

            if self.script:
                try:
                    res = cmd('%s %s %s check' %
                              (self.script, self.server, setting),
                              raiseerrorout=False)

                except CmdError:
                    pass # keep supported == False

                else:
                    res = '\n'.join(res[0])
                    res = res.strip()

                    if res == 'disabled' or res == 'enabled':
                        supported = True

            if not supported:
                if self.script:
                    print "AdminServer: setting %s not supported" % setting
                del self.settings[(mountopt, tag)]
            else:
                print "AdminServer: setting %s supported" % setting

    def has_support(self, mountopt, tag):
        if not self.script:
            return False

        if self.settings.has_key((mountopt, tag)):
            return True

        return False

    def health(self):
        if not self.script:
            return

        res = cmd('%s %s health check' % (self.script, self.server),
                  raiseerrorout=False)
        resstr = '\n'.join(res[0]).strip()

        if resstr:
            # XXX should be an exception?
            print "ServerAdmin Health Warning(s):"
            for line in res[0]:
                print "  %s" % line.strip()
            print "Cannot run tests until this is fixed!"
            sys.exit(1)

    def handle_settings(self, mountopt, tags):
        if not self.script:
            return

        if not tags:
            tags = []

        for tag in TAG_DESC:
            if TAG_DESC[tag][0] != TAGTYPE_SERVER:
                continue

            if self.settings.has_key((mountopt, tag)):
                setting = self.settings[(mountopt, tag)]

                need = 'disabled'
                if tag in tags:
                    need = 'enabled'

                print "ServerAdmin Setting %s -> %s" % (setting, need)

                cmd('%s %s %s %s' % (self.script, self.server, setting, need),
                    raiseerrorout=False)
                res = cmd('%s %s %s check' %
                          (self.script, self.server, setting),
                           raiseerrorout=False)
                res = '\n'.join(res[0])

                if res.strip() != need:
                    raise Exception("ServerAdmin %s: failed to set %s" %
                                (self.script, need))


