"""
Copyright 2012 NetApp, Inc. All Rights Reserved,
contribution by Weston Andros Adamson <dros@netapp.com>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
"""

TAG_DATATYPE_VAL = 1
TAG_DATATYPE_BOOL = 2

TAGTYPE_MOUNTOPT=1
TAGTYPE_SERVER=2
TAGTYPE_CLIENT=3

TAG_VERS='vers'
TAG_MINORVERSION='minorversion'

TAG_DELEG_ENABLED='delegations_enabled'
TAG_PNFS_ENABLED='pnfs_enabled'
TAG_REMOTE='remote'

TAG_DESC = {
    TAG_VERS:          (TAGTYPE_MOUNTOPT, TAG_DATATYPE_VAL),
    TAG_MINORVERSION:  (TAGTYPE_MOUNTOPT, TAG_DATATYPE_VAL),

    TAG_DELEG_ENABLED: (TAGTYPE_SERVER,   TAG_DATATYPE_BOOL),
    TAG_PNFS_ENABLED:  (TAGTYPE_SERVER,   TAG_DATATYPE_BOOL),
    TAG_REMOTE:        (TAGTYPE_SERVER,   TAG_DATATYPE_BOOL),
}

def nfstags(*tags):
    tags = list(set(tags))
    tags.sort()
    return tags

def make_nfsopt(mountopt, tags):
    nfsopt = mountopt

    if tags:
        nfsopt += ',%s' % ','.join(tags)

    return nfsopt

def nfsopt_split(nfsopt):
    split_tags = nfsopt.split(',')

    assert split_tags

    mountopt_tags = []
    server_tags = []
    client_tags = []

    for tag in split_tags:

        if tag.find('=') >= 0:
            name, value = tag.split('=', 1)
        else:
            name = tag
            value = None

        tagtype, datatype = TAG_DESC.get(name,
                                         (TAGTYPE_CLIENT, TAG_DATATYPE_VAL))

        # validate datatype
        if datatype == TAG_DATATYPE_BOOL:
            assert value == None, "BOOL type with value"
        elif datatype == TAG_DATATYPE_VAL:
            assert value, "VAL type with no value: name '%s', value %r" % (name, value)
        else:
            raise NotImplemented

        if tagtype == TAGTYPE_MOUNTOPT:
            mountopt_tags.append(tag)
        elif tagtype == TAGTYPE_SERVER:
            server_tags.append(tag)
        elif tagtype == TAGTYPE_CLIENT:
            client_tags.append(tag)
        else:
            raise NotImplemented

    return (mountopt_tags, server_tags, client_tags)

def nfsopt_to_mountopts_tags(nfsopt):
    mt, st, ct = nfsopt_split(nfsopt)

    mountopts = ','.join(mt)

    tags = []
    tags.extend(st)
    tags.extend(ct)

    return (mountopts, tags)

